
import data
from pygame import image as p_image, error as p_error, mixer as m, get_error


def memoizer(new_dict=dict):
    """
    Creates a memoizer with the given dictionary policy.
 
    The memoizer thus obtained can be used as a decorator to remember the
    results of calls to long-running functions, such as loading images
    from disk. It also means that there will only be one copy of the image,
    which can be quite handy when dealing with restricted resources.
 
    Example:
 
    weak_memoize = memoize(new_dict=weakref.WeakValueDictionary)
 
    @weak_memoize
    def load_image(filename):
        # Your long running image-loading code goes here.
        return result
        
    """
    def memoize(func):
        cache = new_dict()
 
        def memo(*args, **kwargs):
            try:
                # Create a key, resorting to repr if the key isn't hashable.
                try:
                    k = (args, tuple(kwargs.items()))
                    hash(k)
                except TypeError:
                    k = repr(k)
                    
                # Try to return the result from the cache.
                return cache[k]
            except KeyError:
                # The key wasn't found, so invoke the function and save the
                # result in the cache.
                result = func(*args, **kwargs)
                cache[k] = result
                return result
            
        return memo
    
    return memoize
"""
# 'all_sprites.png' is loaded once, by the following line.
man1 = load_image('all_sprites.png', (1, 12, 32, 32), -1)
 
# 'all_sprites.png' is not loaded by this line as it has already
# been loaded once.
man2 = load_image('all_sprites.png', (33, 12, 32, 32), -1)
 
# The following line returns the exact same data as man1, as it
# has the same parameters. The data comes straight from the cache
# and is not reloaded.
man3 = load_image('all_sprites.png', (1, 12, 32, 32), -1)
"""
 
import weakref
memoize = memoizer(new_dict=weakref.WeakValueDictionary)
 
 
@memoize
def load_image(name, a=None):
    """Loads the image with the given name."""
 
    fullname = data.filepath(name)
    try:
        image = p_image.load(fullname)
    except p_error, message:
        print 'Cannot load image:', fullname
        raise SystemExit, message
    if a == -2:
        image = image.convert_alpha()
    elif a == -1:
        colorkey = image.get_at((0,0))
        image.set_colorkey(colorkey)
    else:
        image = image.convert()
    return image

@memoize
class dummysound:
    def play(self): pass

def load_sound(file):
    # if not pygame.mixer:
        # print 'Not pygame.mixer'
        # return dummysound()
    ffile = data.filepath(file)
    try:
        sound = m.Sound(ffile)
        return sound
    except p_error:
        print 'Warning! Cannot load sound "%s", %s'%(ffile, get_error())
        return dummysound()

 
@memoize
def load_sprite(name, colorkey=None, rect=None):
    '''Returns a rectangular portion of a named image.
    "colorkey = -2" for alpha convert'''
    
    image = load_image(name,colorkey)
    if rect is not None:
        image = image.subsurface(Rect(rect))
    return image